@@ -1,5 +1,5 @@
 # coding: utf8
-# Copyright 2021 Lenovo
+# Copyright 2025 Lenovo
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -68,6 +68,7 @@ _healthmap = {
     'Unknown': const.Health.Warning,
     'Warning': const.Health.Warning,
     'OK': const.Health.Ok,
+    None: const.Health.Ok,
 }
 
 
@@ -81,20 +82,6 @@ def _mask_to_cidr(mask):
     return cidr
 
 
-def _to_boolean(attrval):
-    attrval = attrval.lower()
-    if not attrval:
-        return False
-    if ('true'.startswith(attrval) or 'yes'.startswith(attrval)
-            or 'enabled'.startswith(attrval) or attrval == '1'):
-        return True
-    if ('false'.startswith(attrval) or 'no'.startswith(attrval)
-            or 'disabled'.startswith(attrval) or attrval == '0'):
-        return False
-    raise Exception(
-        'Unrecognized candidate for boolean: {0}'.format(attrval))
-
-
 def _cidr_to_mask(cidr):
     return socket.inet_ntop(
         socket.AF_INET, struct.pack(
@@ -127,6 +114,7 @@ def natural_sort(iterable):
 class SensorReading(object):
     def __init__(self, healthinfo, sensor=None, value=None, units=None,
                  unavailable=False):
+        self.states = []
         if sensor:
             self.name = sensor['name']
         else:
@@ -136,12 +124,23 @@ class SensorReading(object):
             self.states = [healthinfo.get('Status', {}).get('Health',
                                                             'Unknown')]
             self.health = _healthmap[healthinfo['Status']['Health']]
-            self.states = [healthinfo['Status']['Health']]
+            if self.health == const.Health.Ok:
+                self.states = []
         self.value = value
         self.state_ids = None
         self.imprecision = None
         self.units = units
         self.unavailable = unavailable
+    
+    def __repr__(self):
+        return repr({
+            'value': self.value,
+            'state_ids': self.state_ids,
+            'units': self.units,
+            'imprecision': self.imprecision,
+            'name': self.name,
+            'unavailable': self.unavailable,
+        })
 
 
 class Command(object):
@@ -164,6 +163,8 @@ class Command(object):
         self._oem = None
         self._gpool = pool
         self._bmcv4ip = None
+        self._bmcv6ip = None
+        self.xauthtoken = None
         for addrinf in socket.getaddrinfo(bmc, 0, 0, socket.SOCK_STREAM):
             if addrinf[0] == socket.AF_INET:
                 self._bmcv4ip = socket.inet_pton(addrinf[0], addrinf[-1][0])
@@ -176,36 +177,76 @@ class Command(object):
         self.wc.set_header('Accept-Encoding', 'gzip')
         self.wc.set_header('OData-Version', '4.0')
         overview = self.wc.grab_json_response('/redfish/v1/')
-        self.wc.set_basic_credentials(userid, password)
         self.username = userid
         self.password = password
+        self.wc.set_basic_credentials(self.username, self.password)
         self.wc.set_header('Content-Type', 'application/json')
-        if 'Systems' not in overview:
+        if 'Systems' not in overview and 'Managers' not in overview:
             raise exc.PyghmiException('Redfish not ready')
-        systems = overview['Systems']['@odata.id']
-        res = self.wc.grab_json_response_with_status(systems)
-        if res[1] == 401:
-            raise exc.PyghmiException('Access Denied')
-        elif res[1] < 200 or res[1] >= 300:
-            raise exc.PyghmiException(repr(res[0]))
-        members = res[0]
+        if 'SessionService' in overview:
+            self._get_session_token(self.wc)
         self._varsensormap = {}
-        systems = members['Members']
-        if sysurl:
-            for system in systems:
-                if system['@odata.id'] == sysurl:
-                    self.sysurl = sysurl
-                    break
+        self.powerurl = None
+        self.sysurl = None
+        if 'Managers' in overview:
+            bmcoll = systems = overview['Managers']['@odata.id']
+            res = self.wc.grab_json_response_with_status(bmcoll)
+            if res[1] == 401:
+                raise exc.PyghmiException('Access Denied')
+            elif res[1] < 200 or res[1] >= 300:
+                raise exc.PyghmiException(repr(res[0]))
+            bmcs = res[0]['Members']
+            if len(bmcs) == 1:
+                self._varbmcurl = bmcs[0]['@odata.id']
+        if 'Systems' in overview:
+            systems = overview['Systems']['@odata.id']
+            res = self.wc.grab_json_response_with_status(systems)
+            if res[1] == 401:
+                raise exc.PyghmiException('Access Denied')
+            elif res[1] < 200 or res[1] >= 300:
+                raise exc.PyghmiException(repr(res[0]))
+            members = res[0]
+            systems = members['Members']
+            if sysurl:
+                for system in systems:
+                    if system['@odata.id'] == sysurl or system['@odata.id'].split('/')[-1] == sysurl:
+                        self.sysurl = system['@odata.id']
+                        break
+                else:
+                    raise exc.PyghmiException(
+                        'Specified sysurl not found: {0}'.format(sysurl))
             else:
-                raise exc.PyghmiException(
-                    'Specified sysurl not found: {0}'.format(sysurl))
-        else:
-            if len(systems) != 1:
-                raise exc.PyghmiException(
-                    'Multi system manager, sysurl is required parameter')
-            self.sysurl = systems[0]['@odata.id']
-        self.powerurl = self.sysinfo.get('Actions', {}).get(
-            '#ComputerSystem.Reset', {}).get('target', None)
+                if len(systems) > 1:
+                    systems = [x for x in systems if 'DPU' not in x['@odata.id']]
+                if len(systems) > 1:
+                    raise exc.PyghmiException(
+                        'Multi system manager, sysurl is required parameter')
+                if len(systems):
+                    self.sysurl = systems[0]['@odata.id']
+                else:
+                    self.sysurl = None
+            self.powerurl = self.sysinfo.get('Actions', {}).get(
+                '#ComputerSystem.Reset', {}).get('target', None)
+
+    def _get_session_token(self, wc):
+        # specification actually indicates we can skip straight to this url
+        username = self.username
+        password = self.password
+        if not isinstance(username, str):
+            username = username.decode()
+        if not isinstance(password, str):
+            password = password.decode()
+        rsp = wc.grab_rsp('/redfish/v1/SessionService/Sessions',
+                          {'UserName': username, 'Password': password})
+        rsp.read()
+        self.xauthtoken = rsp.getheader('X-Auth-Token')
+        if self.xauthtoken:
+            if 'Authorization' in wc.stdheaders:
+                del wc.stdheaders['Authorization']
+            if 'Authorization' in self.wc.stdheaders:
+                del self.wc.stdheaders['Authorization']
+            wc.stdheaders['X-Auth-Token'] = self.xauthtoken
+            self.wc.stdheaders['X-Auth-Token'] = self.xauthtoken
 
     @property
     def _accountserviceurl(self):
@@ -374,8 +415,11 @@ class Command(object):
             Operator, and ReadOnly
         """
         accinfo = self._account_url_info_by_id(uid)
-        if not accinfo:
-            raise Exception("Unable to find indicated uid")
+        if accinfo:
+            method = 'PATCH'
+        else:
+            accinfo = (self._accountserviceurl + '/Accounts', {})
+            method = 'POST'
         if privilege_level.startswith('custom.'):
             privilege_level = privilege_level.replace('custom.', '')
         for role in self._validroles:
@@ -388,9 +432,18 @@ class Command(object):
             "Password": password,
             "RoleId": privilege_level,
         }
-        self._do_web_request(accinfo[0], userinfo, method='PATCH', etag=etag)
+        self._do_web_request(accinfo[0], userinfo, method=method, etag=etag)
         return True
 
+    def get_screenshot(self, outfile):
+        return self.oem.get_screenshot(outfile)
+
+    def get_ikvm_methods(self):
+        return self.oem.get_ikvm_methods()
+
+    def get_ikvm_launchdata(self):
+        return self.oem.get_ikvm_launchdata()
+
     def user_delete(self, uid):
         self.oem.user_delete(uid)
 
@@ -432,7 +485,13 @@ class Command(object):
 
     @property
     def sysinfo(self):
-        return self._do_web_request(self.sysurl)
+        if not self.sysurl:
+            return {}
+        try:
+            return self._do_web_request(self.sysurl)
+        except exc.RedfishError:
+            self.sysurl = None
+            return {}
 
     @property
     def bmcinfo(self):
@@ -466,10 +525,8 @@ class Command(object):
             raise exc.InvalidParameterValue(
                 "Unknown power state %s requested" % powerstate)
         powerstate = powerstates[powerstate]
-        result = self.wc.grab_json_response_with_status(
+        self._do_web_request(
             self.powerurl, {'ResetType': powerstate})
-        if result[1] < 200 or result[1] >= 300:
-            raise exc.PyghmiException(result[0])
         if wait and reqpowerstate in ('on', 'off', 'softoff', 'shutdown'):
             if reqpowerstate in ('softoff', 'shutdown'):
                 reqpowerstate = 'off'
@@ -506,6 +563,8 @@ class Command(object):
     def _do_web_request(self, url, payload=None, method=None, cache=True,
                         etag=None):
         res = None
+        if cache is True:
+            cache = 30
         if cache and payload is None and method is None:
             res = self._get_cache(url, cache)
         if res:
@@ -519,6 +578,17 @@ class Command(object):
         finally:
             if 'If-Match' in wc.stdheaders:
                 del wc.stdheaders['If-Match']
+        if res[1] == 401 and self.xauthtoken:
+            wc.set_basic_credentials(self.username, self.password)
+            self._get_session_token(wc)
+            if etag:
+                wc.stdheaders['If-Match'] = etag
+            try:
+                res = wc.grab_json_response_with_status(url, payload,
+                                                        method=method)
+            finally:
+                if 'If-Match' in wc.stdheaders:
+                    del wc.stdheaders['If-Match']
         if res[1] < 200 or res[1] >= 300:
             try:
                 info = json.loads(res[0])
@@ -532,7 +602,7 @@ class Command(object):
                 msgid = ','.join(msgid)
                 raise exc.RedfishError(errmsg, msgid=msgid)
             except (ValueError, KeyError):
-                raise exc.PyghmiException(str(url) + ":" + res[0])
+                raise exc.PyghmiException(str(url) + ":" + str(res[0]))
         if payload is None and method is None:
             self._urlcache[url] = {'contents': res[0],
                                    'vintage': os.times()[4]}
@@ -618,37 +688,66 @@ class Command(object):
     @property
     def _sensormap(self):
         if not self._varsensormap:
-            for chassis in self.sysinfo.get('Links', {}).get('Chassis', []):
-                self._mapchassissensors(chassis)
+            if self.sysinfo:
+                for chassis in self.sysinfo.get('Links', {}).get('Chassis', []):
+                    self._mapchassissensors(chassis)
+            else:  # no system, but check if this is a singular chassis
+                rootinfo = self._do_web_request('/redfish/v1/')
+                chassiscol = rootinfo.get('Chassis', {}).get('@odata.id', '')
+                if chassiscol:
+                    chassislist = self._do_web_request(chassiscol)
+                    if len(chassislist.get('Members', [])) == 1:
+                        self._mapchassissensors(chassislist['Members'][0])
         return self._varsensormap
 
     def _mapchassissensors(self, chassis):
         chassisurl = chassis['@odata.id']
         chassisinfo = self._do_web_request(chassisurl)
-        powurl = chassisinfo.get('Power', {}).get('@odata.id', '')
-        if powurl:
-            powinf = self._do_web_request(powurl)
-            for voltage in powinf.get('Voltages', []):
-                if 'Name' in voltage:
-                    self._varsensormap[voltage['Name']] = {
-                        'name': voltage['Name'], 'url': powurl,
-                        'type': 'Voltage'}
-        thermurl = chassisinfo.get('Thermal', {}).get('@odata.id', '')
-        if thermurl:
-            therminf = self._do_web_request(thermurl)
-            for fan in therminf.get('Fans', []):
-                if 'Name' in fan:
-                    self._varsensormap[fan['Name']] = {
-                        'name': fan['Name'], 'type': 'Fan',
-                        'url': thermurl}
-            for temp in therminf.get('Temperatures', []):
-                if 'Name' in temp:
-                    self._varsensormap[temp['Name']] = {
-                        'name': temp['Name'], 'type': 'Temperature',
-                        'url': thermurl}
+        sensors = None
+        if self.oem.usegenericsensors:
+            sensors = chassisinfo.get('Sensors', {}).get('@odata.id', '')
+        if sensors:
+            sensorinf = self._do_web_request(sensors)
+            for sensor in sensorinf.get('Members', []):
+                sensedata = self._do_web_request(sensor['@odata.id'])
+                if 'Name' in sensedata:
+                    sensetype = sensedata.get('ReadingType', 'Unknown')
+                    self._varsensormap[sensedata['Name']] = {
+                        'name': sensedata['Name'], 'type': sensetype,
+                        'url': sensor['@odata.id'], 'generic': True}
+        else:
+            powurl = chassisinfo.get('Power', {}).get('@odata.id', '')
+            if powurl:
+                powinf = self._do_web_request(powurl)
+                for voltage in powinf.get('Voltages', []):
+                    if 'Name' in voltage:
+                        self._varsensormap[voltage['Name']] = {
+                            'name': voltage['Name'], 'url': powurl,
+                            'type': 'Voltage'}
+            thermurl = chassisinfo.get('Thermal', {}).get('@odata.id', '')
+            if thermurl:
+                therminf = self._do_web_request(thermurl)
+                for fan in therminf.get('Fans', []):
+                    if 'Name' in fan:
+                        self._varsensormap[fan['Name']] = {
+                            'name': fan['Name'], 'type': 'Fan',
+                            'url': thermurl}
+                for temp in therminf.get('Temperatures', []):
+                    if 'Name' in temp:
+                        self._varsensormap[temp['Name']] = {
+                            'name': temp['Name'], 'type': 'Temperature',
+                            'url': thermurl}
         for subchassis in chassisinfo.get('Links', {}).get('Contains', []):
             self._mapchassissensors(subchassis)
 
+    def _get_thermals(self, chassis):
+        chassisurl = chassis['@odata.id']
+        chassisinfo = self._do_web_request(chassisurl)
+        thermurl = chassisinfo.get('Thermal', {}).get('@odata.id', '')
+        if thermurl:
+            therminf = self._do_web_request(thermurl, cache=1)
+            return therminf.get('Temperatures', [])
+
     @property
     def _bmcurl(self):
         if not self._varbmcurl:
@@ -748,8 +847,20 @@ class Command(object):
         self._do_web_request(url, {'ResetType': action})
 
     def set_identify(self, on=True, blink=None):
+        if hasattr(self.oem, 'set_identify'):
+            return self.oem.set_identify(on, blink)
+        targurl = self.sysurl
+        if not targurl:
+            root = self._do_web_request('/redfish/v1')
+            systemsurl = root.get('Systems', {}).get('@odata.id', None)
+            if systemsurl:
+                targurl = self._do_web_request(systemsurl)
+                if len(targurl.get('Members', [])) == 1:
+                    targurl = targurl['Members'][0]['@odata.id']
+        if not targurl:
+            raise Exception("Unable to identify system url")
         self._do_web_request(
-            self.sysurl,
+            targurl,
             {'IndicatorLED': 'Blinking' if blink else 'Lit' if on else 'Off'},
             method='PATCH', etag='*')
 
@@ -764,72 +875,10 @@ class Command(object):
         return {'identifystate': self._idstatemap[ledstate]}
 
     def get_health(self, verbose=True):
-        health = self.sysinfo.get('Status', {})
-        health = health.get('HealthRollup', health.get('Health', 'Unknown'))
-        warnunknown = health == 'Unknown'
-        health = _healthmap[health]
-        summary = {'badreadings': [], 'health': health}
-        if health > 0 and verbose:
-            # now have to manually peruse all psus, fans, processors, ram,
-            # storage
-            procsumstatus = self.sysinfo.get('ProcessorSummary', {}).get(
-                'Status', {})
-            procsumstatus = procsumstatus.get('HealthRollup',
-                                              procsumstatus.get('Health',
-                                                                None))
-            if procsumstatus != 'OK':
-                procfound = False
-                procurl = self.sysinfo.get('Processors', {}).get('@odata.id',
-                                                                 None)
-                if procurl:
-                    for cpu in self._do_web_request(procurl).get(
-                            'Members', []):
-                        cinfo = self._do_web_request(cpu['@odata.id'])
-                        if cinfo.get('Status', {}).get(
-                                'State', None) == 'Absent':
-                            continue
-                        if cinfo.get('Status', {}).get(
-                                'Health', None) not in ('OK', None):
-                            procfound = True
-                            summary['badreadings'].append(SensorReading(cinfo))
-                if not procfound:
-                    procinfo = self.sysinfo['ProcessorSummary']
-                    procinfo['Name'] = 'Processors'
-                    summary['badreadings'].append(SensorReading(procinfo))
-            memsumstatus = self.sysinfo.get(
-                'MemorySummary', {}).get('Status', {})
-            memsumstatus = memsumstatus.get('HealthRollup',
-                                            memsumstatus.get('Health', None))
-            if memsumstatus != 'OK':
-                dimmfound = False
-                for mem in self._do_web_request(
-                        self.sysinfo['Memory']['@odata.id'])['Members']:
-                    dimminfo = self._do_web_request(mem['@odata.id'])
-                    if dimminfo.get('Status', {}).get(
-                            'State', None) == 'Absent':
-                        continue
-                    if dimminfo.get('Status', {}).get(
-                            'Health', None) not in ('OK', None):
-                        summary['badreadings'].append(SensorReading(dimminfo))
-                        dimmfound = True
-                if not dimmfound:
-                    meminfo = self.sysinfo['MemorySummary']
-                    meminfo['Name'] = 'Memory'
-                    summary['badreadings'].append(SensorReading(meminfo))
-            for adapter in self.sysinfo['PCIeDevices']:
-                adpinfo = self._do_web_request(adapter['@odata.id'])
-                if adpinfo['Status']['Health'] not in ('OK', None):
-                    summary['badreadings'].append(SensorReading(adpinfo))
-            for fun in self.sysinfo['PCIeFunctions']:
-                funinfo = self._do_web_request(fun['@odata.id'])
-                if funinfo['Status']['Health'] not in ('OK', None):
-                    summary['badreadings'].append(SensorReading(funinfo))
-        if warnunknown and not summary['badreadings']:
-            unkinf = SensorReading({'Name': 'BMC',
-                                    'Status': {'Health': 'Unknown'}})
-            unkinf.states = ['System does not provide health information']
-            summary['badreadings'].append(unkinf)
-        return summary
+        return self.oem.get_health(self, verbose)
+
+    def get_extended_bmc_configuration(self, hideadvanced=True):
+        return self.oem.get_extended_bmc_configuration(self, hideadvanced=hideadvanced)
 
     def get_bmc_configuration(self):
         """Get miscellaneous BMC configuration
@@ -858,6 +907,54 @@ class Command(object):
     def set_system_configuration(self, changeset):
         return self.oem.set_system_configuration(changeset, self)
 
+    def get_ntp_enabled(self):
+        bmcinfo = self._do_web_request(self._bmcurl)
+        netprotocols = bmcinfo.get('NetworkProtocol', {}).get('@odata.id', None)
+        if netprotocols:
+            netprotoinfo = self._do_web_request(netprotocols)
+            enabled = netprotoinfo.get('NTP', {}).get('ProtocolEnabled', False)
+            return enabled
+        return False
+
+    def set_ntp_enabled(self, enable):
+        bmcinfo = self._do_web_request(self._bmcurl)
+        netprotocols = bmcinfo.get('NetworkProtocol', {}).get('@odata.id', None)
+        if netprotocols:
+            request = {'NTP':{'ProtocolEnabled': enable}}
+            self._do_web_request(netprotocols, request, method='PATCH')
+            self._do_web_request(netprotocols, cache=0)
+
+    def get_ntp_servers(self):
+        bmcinfo = self._do_web_request(self._bmcurl)
+        netprotocols = bmcinfo.get('NetworkProtocol', {}).get('@odata.id', None)
+        if not netprotocols:
+            return []
+        netprotoinfo = self._do_web_request(netprotocols)
+        return netprotoinfo.get('NTP', {}).get('NTPServers', [])
+
+    def set_ntp_server(self, server, index=None):
+        bmcinfo = self._do_web_request(self._bmcurl)
+        netprotocols = bmcinfo.get('NetworkProtocol', {}).get('@odata.id', None)
+        currntpservers = self.get_ntp_servers()
+        if index is None:
+            if server in currntpservers:
+                return
+            currntpservers = [server] + currntpservers
+        else:
+            if (index + 1) > len(currntpservers):
+                if not server:
+                    return
+                currntpservers.append(server)
+            else:
+                if not server:
+                    del currntpservers[index]
+                else:
+                    currntpservers[index] = server
+        request = {'NTP':{'NTPServers': currntpservers}}
+        self._do_web_request(netprotocols, request, method='PATCH')
+        self._do_web_request(netprotocols, cache=0)
+
+
     def clear_bmc_configuration(self):
         """Reset BMC to factory default
 
@@ -865,8 +962,16 @@ class Command(object):
         In many cases, this may render remote network access impracticle or
         impossible."
         """
+        bmcinfo = self._do_web_request(self._bmcurl)
+        rc = bmcinfo.get('Actions', {}).get('#Manager.ResetToDefaults', {})
+        actinf = rc.get('ResetType@Redfish.AllowableValues', [])
+        if 'ResetAll' in actinf:
+            acturl = rc.get('target', None)
+            if acturl:
+                self._do_web_request(acturl, {'ResetType': 'ResetAll'})
+                return
         raise exc.UnsupportedFunctionality(
-            'Clear BMC configuration not supported in redfish yet')
+            'Clear BMC configuration not supported on this platform')
 
     def get_system_configuration(self, hideadvanced=True):
         return self.oem.get_system_configuration(hideadvanced, self)
@@ -1007,14 +1112,14 @@ class Command(object):
                              {'HostName': hostname}, 'PATCH')
 
     def get_firmware(self, components=()):
+        self._fwnamemap = {}
         try:
-            for firminfo in self.oem.get_firmware_inventory(components):
+            for firminfo in self.oem.get_firmware_inventory(components, self):
                 yield firminfo
         except exc.BypassGenericBehavior:
             return
         fwlist = self._do_web_request(self._fwinventory)
         fwurls = [x['@odata.id'] for x in fwlist.get('Members', [])]
-        self._fwnamemap = {}
         for res in self._do_bulk_requests(fwurls):
             res = self._extract_fwinfo(res)
             if res[0] is None:
@@ -1022,7 +1127,7 @@ class Command(object):
             yield res
 
     def _extract_fwinfo(self, inf):
-        currinf = {}
+        currinf = self._oem._extract_fwinfo(inf)
         fwi, url = inf
         fwname = fwi.get('Name', 'Unknown')
         if fwname in self._fwnamemap:
@@ -1036,6 +1141,7 @@ class Command(object):
         currinf['id'] = fwi.get('Id', None)
         currinf['version'] = fwi.get('Version', 'Unknown')
         currinf['date'] = parse_time(fwi.get('ReleaseDate', ''))
+        currinf['software_id'] = fwi.get('SoftwareId', '')
         if not (currinf['version'] or currinf['date']):
             return None, None
         # TODO(Jarrod Johnson): OEM extended data with buildid
@@ -1117,80 +1223,57 @@ class Command(object):
     @property
     def oem(self):
         if not self._oem:
+            if self.sysurl:
+                self._do_web_request(self.sysurl, cache=False)  # This is to trigger token validation and renewel
+            elif self._varbmcurl:
+                self._do_web_request(self._varbmcurl, cache=False)  # This is to trigger token validation and renewel
             self._oem = oem.get_oem_handler(
                 self.sysinfo, self.sysurl, self.wc, self._urlcache, self)
             self._oem.set_credentials(self.username, self.password)
         return self._oem
 
     def get_description(self):
-        return self.oem.get_description()
+        return self.oem.get_description(self)
 
     def get_event_log(self, clear=False):
-        bmcinfo = self._do_web_request(self._bmcurl)
-        lsurl = bmcinfo.get('LogServices', {}).get('@odata.id', None)
-        if not lsurl:
-            return
-        currtime = bmcinfo.get('DateTime', None)
-        correction = timedelta(0)
-        utz = tz.tzoffset('', 0)
-        ltz = tz.gettz()
-        if currtime:
-            currtime = parse_time(currtime)
-        if currtime:
-            now = datetime.now(utz)
-            try:
-                correction = now - currtime
-            except TypeError:
-                correction = now - currtime.replace(tzinfo=utz)
-        lurls = self._do_web_request(lsurl).get('Members', [])
-        for lurl in lurls:
-            lurl = lurl['@odata.id']
-            loginfo = self._do_web_request(lurl, cache=(not clear))
-            entriesurl = loginfo.get('Entries', {}).get('@odata.id', None)
-            if not entriesurl:
-                continue
-            logid = loginfo.get('Id', '')
-            entries = self._do_web_request(entriesurl, cache=False)
-            if clear:
-                # The clear is against the log service etag, not entries
-                # so we have to fetch service etag after we fetch entries
-                # until we can verify that the etag is consistent to prove
-                # that the clear is atomic
-                newloginfo = self._do_web_request(lurl, cache=False)
-                clearurl = newloginfo.get('Actions', {}).get(
-                    '#LogService.ClearLog', {}).get('target', '')
-                while clearurl:
-                    try:
-                        self._do_web_request(clearurl, method='POST',
-                                             payload={})
-                        clearurl = False
-                    except exc.PyghmiException as e:
-                        if 'EtagPreconditionalFailed' not in str(e):
-                            raise
-                        # This doesn't guarantee atomicity, but it mitigates
-                        # greatly.  Unfortunately some implementations
-                        # mutate the tag endlessly and we have no hope
-                        entries = self._do_web_request(entriesurl, cache=False)
-                        newloginfo = self._do_web_request(lurl, cache=False)
-            for log in entries.get('Members', []):
-                if ('Created' not in log and 'Message' not in log
-                        and 'Severity' not in log):
-                    # without any data, this log entry isn't actionable
-                    continue
-                record = {}
-                record['log_id'] = logid
-                parsedtime = parse_time(log.get('Created', ''))
-                if parsedtime:
-                    entime = parsedtime + correction
-                    entime = entime.astimezone(ltz)
-                    record['timestamp'] = entime.strftime('%Y-%m-%dT%H:%M:%S')
-                else:
-                    record['timestamp'] = log.get('Created', '')
-                record['message'] = log.get('Message', None)
-                record['severity'] = _healthmap.get(
-                    log.get('Severity', 'Warning'), const.Health.Ok)
-                yield record
+        return self.oem.get_event_log(clear, self)
 
+    def _get_chassis_env(self, chassis):
+        chassisurl = chassis['@odata.id']
+        chassisinfo = self._do_web_request(chassisurl)
+        envurl = chassisinfo.get('EnvironmentMetrics', {}).get('@odata.id', '')
+        if not envurl:
+            return {}
+        envmetric = self._do_web_request(envurl, cache=1)
+        retval = {
+            'watts': envmetric.get('PowerWatts', {}).get('Reading', None),
+            'inlet': envmetric.get('TemperatureCelsius', {}).get('Reading', None)
+        }
+        return retval
+
+    def get_average_processor_temperature(self):
+        return self.oem.get_average_processor_temperature(self)
+
+
+    def get_system_power_watts(self):
+        return self.oem.get_system_power_watts(self)
+
+    def get_inlet_temperature(self):
+        inlets = []
+        for chassis in self.sysinfo.get('Links', {}).get('Chassis', []):
+            envinfo = self._get_chassis_env(chassis)
+            currinlet = envinfo.get('inlet', None)
+            if currinlet:
+                inlets.append(currinlet)
+        if inlets:
+            val = sum(inlets) / len(inlets)
+            unavail = False
+        else:
+            val = None
+            unavail = True
+        return SensorReading(
+                        None, {'name': 'Inlet Temperature'}, value=val, units='°C',
+                        unavailable=unavail)
     def get_sensor_descriptions(self):
         for sensor in natural_sort(self._sensormap):
             yield self._sensormap[sensor]
@@ -1207,6 +1290,16 @@ class Command(object):
             yield self.get_sensor_reading(sensor)
 
     def _extract_reading(self, sensor, reading):
+        if sensor.get('generic', False):  # generic sensor
+            val = reading.get('Reading', None)
+            unavail = val is None
+            units = reading.get('ReadingUnits', None)
+            if units == 'Cel':
+                units = '°C'
+            if units == 'cft_i/min':
+                units = 'CFM'
+            return SensorReading(reading, None, value=val, units=units,
+                          unavailable=unavail)
         if sensor['type'] == 'Fan':
             for fan in reading['Fans']:
                 if fan['Name'] == sensor['name']:
@@ -1267,6 +1360,16 @@ class Command(object):
         """
         return self.oem.apply_storage_configuration(cfgspec)
 
+    def check_storage_configuration(self, cfgspec=None):
+        """Evaluate a configuration for validity
+
+        This will check if configuration is currently available and, if given,
+        whether the specified cfgspec can be applied.
+        :param cfgspec: A pyghmi.storage.ConfigSpec describing desired oonfig
+        :return:
+        """
+        return self._oem.check_storage_configuration(cfgspec)
+
     def attach_remote_media(self, url, username=None, password=None):
         """Attach remote media by url
 
@@ -1292,9 +1395,18 @@ class Command(object):
         if vmcoll:
             vmlist = self._do_web_request(vmcoll)
             vmurls = [x['@odata.id'] for x in vmlist.get('Members', [])]
+        suspendedxauth = False
+        if 'X-Auth-Token' in self.wc.stdheaders:
+            suspendedxauth = True
+            del self.wc.stdheaders['X-Auth-Token']
+            self.wc.set_basic_credentials(self.username, self.password)
         try:
             self.oem.attach_remote_media(url, username, password, vmurls)
         except exc.BypassGenericBehavior:
+            if suspendedxauth:
+                self.wc.stdheaders['X-Auth-Token'] = self.xauthtoken
+                if 'Authorization' in self.wc.stdheaders:
+                    del self.wc.stdheaders['Authorization']
             return
         for vmurl in vmurls:
             vminfo = self._do_web_request(vmurl, cache=False)
@@ -1316,6 +1428,12 @@ class Command(object):
                     else:
                         raise
             break
+        if suspendedxauth:
+                self.wc.stdheaders['X-Auth-Token'] = self.xauthtoken
+                if 'Authorization' in self.wc.stdheaders:
+                    del self.wc.stdheaders['Authorization']
+        for res in self.oem.list_media(self, cache=False):
+            pass
 
     def detach_remote_media(self):
         try:
@@ -1349,6 +1467,8 @@ class Command(object):
                                                      method='PATCH')
                             else:
                                 raise
+        for res in self.oem.list_media(self, cache=False):
+            pass
 
     def upload_media(self, filename, progress=None, data=None):
         """Upload a file to be hosted on the target BMC
@@ -1363,6 +1483,9 @@ class Command(object):
         """
         return self.oem.upload_media(filename, progress, data)
 
+    def get_update_status(self):
+        return self.oem.get_update_status()
+
     def update_firmware(self, file, data=None, progress=None, bank=None):
         """Send file to BMC to perform firmware update
 
@@ -1385,23 +1508,24 @@ class Command(object):
         return self.oem.get_diagnostic_data(savefile, progress, autosuffix)
 
     def get_licenses(self):
-        return self.oem.get_licenses()
+        return self.oem.get_licenses(self)
 
     def delete_license(self, name):
-        return self.oem.delete_license(name)
+        return self.oem.delete_license(name, self)
 
     def save_licenses(self, directory):
         if os.path.exists(directory) and not os.path.isdir(directory):
             raise exc.InvalidParameterValue(
                 'Not allowed to overwrite existing file: {0}'.format(
                     directory))
-        return self.oem.save_licenses(directory)
+        return self.oem.save_licenses(directory, self)
 
     def apply_license(self, filename, progress=None, data=None):
-        return self.oem.apply_license(filename, progress, data)
+        return self.oem.apply_license(filename, self, progress, data)
 
 
 if __name__ == '__main__':
     print(repr(
         Command(sys.argv[1], os.environ['BMCUSER'], os.environ['BMCPASS'],
                 verifycallback=lambda x: True).get_power()))
+
